/*
 C-level
*/
#pragma BEGINDUMP

#include <mgdefs.h>
#include <windows.h>
#include "hbapi.h"

typedef UINT (CALLBACK* MSIMG32GradientFill)(HDC,CONST PTRIVERTEX,DWORD,CONST PVOID,DWORD,DWORD);

HB_FUNC( FILLGRADIENTEX )
{
   HINSTANCE m_hMsimg32;
   MSIMG32GradientFill m_dllGradientFillFunc;
   TRIVERTEX rcVertex[4];
   GRADIENT_TRIANGLE gTri[2];

   BOOL bPaintedGradient = FALSE;

   m_hMsimg32 = LoadLibrary( "msimg32.dll" );
   if ( m_hMsimg32 != NULL )
   {
     m_dllGradientFillFunc = ( MSIMG32GradientFill ) GetProcAddress( m_hMsimg32, "GradientFill" );
     if ( m_dllGradientFillFunc != NULL )
     {
       rcVertex[0].x = hb_parvni(2, 1);
       rcVertex[0].y = hb_parvni(2, 2);
       rcVertex[0].Red   = (USHORT) (hb_parvni(2, 3) << 8);
       rcVertex[0].Green = (USHORT) (hb_parvni(2, 4) << 8);
       rcVertex[0].Blue  = (USHORT) (hb_parvni(2, 5) << 8);
       rcVertex[0].Alpha = 0;

       rcVertex[1].x = hb_parvni(3, 1);
       rcVertex[1].y = hb_parvni(3, 2);
       rcVertex[1].Red   = (USHORT) (hb_parvni(3, 3) << 8);
       rcVertex[1].Green = (USHORT) (hb_parvni(3, 4) << 8);
       rcVertex[1].Blue  = (USHORT) (hb_parvni(3, 5) << 8);
       rcVertex[1].Alpha = 0;

       rcVertex[2].x = hb_parvni(4, 1);
       rcVertex[2].y = hb_parvni(4, 2);
       rcVertex[2].Red   = (USHORT) (hb_parvni(4, 3) << 8);
       rcVertex[2].Green = (USHORT) (hb_parvni(4, 4) << 8);
       rcVertex[2].Blue  = (USHORT) (hb_parvni(4, 5) << 8);
       rcVertex[2].Alpha = 0;

       rcVertex[3].x = 0;
       rcVertex[3].y = 0;
       rcVertex[3].Red   = 0;
       rcVertex[3].Green = 0;
       rcVertex[3].Blue  = 0;
       rcVertex[3].Alpha = 0;

       gTri[0].Vertex1 = 0;  // first triangle
       gTri[0].Vertex2 = 1;
       gTri[0].Vertex3 = 2;

       gTri[1].Vertex1 = 0;  // second triangle
       gTri[1].Vertex2 = 2;
       gTri[1].Vertex3 = 3;

       bPaintedGradient = m_dllGradientFillFunc
         ( (HDC) hb_parnl( 1 ), rcVertex, 4, &gTri, 2,
         GRADIENT_FILL_TRIANGLE );
     }
     FreeLibrary( m_hMsimg32 );
   }

   hb_retl( bPaintedGradient );
}


HB_FUNC( GRAGRADIENTEX )
{
   HINSTANCE m_hMsimg32;
   MSIMG32GradientFill m_dllGradientFillFunc;
   TRIVERTEX rcVertex[4];
   GRADIENT_TRIANGLE gTri[2];

   BOOL bPaintedGradient = FALSE;

   m_hMsimg32 = LoadLibrary( "msimg32.dll" );
   if ( m_hMsimg32 != NULL )
   {
     m_dllGradientFillFunc = ( MSIMG32GradientFill ) GetProcAddress( m_hMsimg32, "GradientFill" );
     if ( m_dllGradientFillFunc != NULL )
     {

       rcVertex[0].x = hb_parvni(2, 1);
       rcVertex[0].y = hb_parvni(2, 2);
       rcVertex[0].Red   = (USHORT) (hb_parvni(2, 3) << 8);
       rcVertex[0].Green = (USHORT) (hb_parvni(2, 4) << 8);
       rcVertex[0].Blue  = (USHORT) (hb_parvni(2, 5) << 8);
       rcVertex[0].Alpha = 0;

       rcVertex[1].x = hb_parvni(3, 1);
       rcVertex[1].y = hb_parvni(3, 2);
       rcVertex[1].Red   = (USHORT) (hb_parvni(3, 3) << 8);
       rcVertex[1].Green = (USHORT) (hb_parvni(3, 4) << 8);
       rcVertex[1].Blue  = (USHORT) (hb_parvni(3, 5) << 8);
       rcVertex[1].Alpha = 0;

       rcVertex[2].x = hb_parvni(4, 1);
       rcVertex[2].y = hb_parvni(4, 2);
       rcVertex[2].Red   = (USHORT) (hb_parvni(4, 3) << 8);
       rcVertex[2].Green = (USHORT) (hb_parvni(4, 4) << 8);
       rcVertex[2].Blue  = (USHORT) (hb_parvni(4, 5) << 8);
       rcVertex[2].Alpha = 0;

       gTri[0].Vertex1 = 0;  // first triangle
       gTri[0].Vertex2 = 2;
       gTri[0].Vertex3 = 1;

       bPaintedGradient = m_dllGradientFillFunc
         ( (HDC) hb_parnl( 1 ), rcVertex, 3, &gTri, 1,
         GRADIENT_FILL_TRIANGLE );
     }
     FreeLibrary( m_hMsimg32 );
   }

   hb_retl( bPaintedGradient );
}


HB_FUNC( GETPIXELCOLOR )
{
  COLORREF pixel, C1, C2, C3;
  BOOL result;

  pixel = GetPixel( ( HDC ) hb_parnl( 1 ), hb_parnl( 2 ), hb_parnl( 3 ) );

  result = ( pixel != CLR_INVALID ? HB_TRUE : HB_FALSE );
  if ( result )
  {
    C1 = ( USHORT ) ( GetRValue( pixel ) );
    C2 = ( USHORT ) ( GetGValue( pixel ) );
    C3 = ( USHORT ) ( GetBValue( pixel ) );
    HB_STORNI( C1, 4, 1);
    HB_STORNI( C2, 4, 2);
    HB_STORNI( C3, 4, 3);
  }

  hb_retl( result );
}

HB_FUNC( SETPIXEL )
{
   hb_retnl( (ULONG) SetPixel( (HDC) hb_parnl( 1 ) ,
            hb_parnl( 2 )      ,
            hb_parnl( 3 )      ,
         (COLORREF) hb_parnl( 4 ) ) );
}

HB_FUNC( BEGINPAINT )
{
   HWND hWnd = ( HWND ) ( LONG_PTR ) HB_PARNL( 1 );

   if( IsWindow( hWnd ) )
   {
      PAINTSTRUCT ps;

      HB_RETNL( ( LONG_PTR ) BeginPaint( hWnd, &ps ) );

      hb_storclen( ( const char * ) &ps, sizeof( PAINTSTRUCT ), 2 );
   }
   else
      HB_RETNL( ( LONG_PTR ) NULL );
}

HB_FUNC( ENDPAINT )
{
   HWND hWnd         = ( HWND ) ( LONG_PTR ) HB_PARNL( 1 );
   PAINTSTRUCT * pps = ( PAINTSTRUCT * ) hb_parc( 2 );

   if( IsWindow( hWnd ) && pps )
      hb_retl( EndPaint( hWnd, pps ) );
   else
      hb_retl( HB_FALSE );
}

#pragma ENDDUMP